import 'dart:async';
import 'dart:developer';
import 'dart:io';

import 'package:device_info_plus/device_info_plus.dart';
import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import '../../utils/utils.dart';
import '../model/response/account_info_response.dart';
import '../model/response/create_session_response.dart';
import '../model/response/process_response.dart';
import '../repository/soliq_id_repository.dart';

part 'soliq_id_event.dart';

part 'soliq_id_state.dart';

class SoliqIdBloc extends Bloc<SoliqIdEvent, SoliqIdState> {
  SoliqIdRepository repository;
  String deviceName = '';
  String identifier = '';
  String brand = '';
  int osType = 0;
  String osVersion = "";
  final DeviceInfoPlugin deviceInfoPlugin = DeviceInfoPlugin();
  String? token;

  SoliqIdBloc(this.repository) : super(SoliqIdInitial()) {
    on<SoliqIdAccountInfoEvent>(_onAccountInfo);
    on<SoliqIdCreateSessionEvent>(_onCreateSession);
    on<SoliqIdProcessEvent>(_onSetProcess);
    on<SoliqIdCompareFaceEvent>(_onSetCompareFace);
  }

  Future<void> getDevice() async {
    if (Platform.isAndroid) {
      var build = await deviceInfoPlugin.androidInfo;
      deviceName = build.model;
      identifier = build.model;
      brand = build.brand;
      osVersion = build.id;
    } else if (Platform.isIOS) {
      var data = await deviceInfoPlugin.iosInfo;
      deviceName = data.name;
      identifier = data.identifierForVendor ?? "";
      brand = data.utsname.nodename;
      osVersion = data.systemVersion;
    }
  }

  FutureOr<void> _onAccountInfo(
      SoliqIdAccountInfoEvent event, Emitter<SoliqIdState> emit) async {
    try {
      emit(SoliqIdPinflLoadingState());
      final response = await repository.setAccountInfo(
        deviceId: "string",
        projectId: event.projectId,
        deviceName: "string",
        factoryName: "string",
        osType: 0,
        pinfl: event.pinfl,
        osVersion: "string",
        appVersion: "1.0.1",
        firebaseId: "string",
        lang: "string",
      );
      emit(SoliqIdAccountInfoSuccessState(
        response: response,
      ));
    } on Object catch (e) {
      log(e.toString());
    }
  }

  FutureOr<void> _onCreateSession(
      SoliqIdCreateSessionEvent event, Emitter<SoliqIdState> emit) async {
    try {
      emit(SoliqIdPinflLoadingState());
      final response = await repository.createSession(
        deviceId: "string",
        projectId: event.projectId,
        deviceName: "string",
        factoryName: "string",
        osType: 0,
        pinfl: event.pinfl,
        osVersion: "string",
        appVersion: "1.0.1",
        firebaseId: "string",
        lang: "string",
        fio: event.fio,
      );
      emit(
        SoliqIdCreateSessionSuccessState(
          response: response,
        ),
      );
    } on Object catch (e) {
      log(e.toString());
    }
  }

  FutureOr<void> _onSetProcess(
      SoliqIdProcessEvent event, Emitter<SoliqIdState> emit) async {
    try {
      emit(SoliqIdPinflLoadingState());
      final response = await repository.setProcess(
        deviceId: await Utils.getDeviceId(),
        id: event.id,
        asked: event.asked,
        processId: event.processId,
        date: '',
      );
      emit(SoliqIdProcessSuccessState(
        response: response,
      ));
    } on Object catch (e) {
      log(e.toString());
    }
  }

  FutureOr<void> _onSetCompareFace(
      SoliqIdCompareFaceEvent event, Emitter<SoliqIdState> emit) async {
    try {
      emit(SoliqIdCompareFaceLoadingState());
      final response = await repository.compareFace(
        deviceId: await Utils.getDeviceId(),
        id: event.id,
        asked: event.asked,
        processId: event.processId,
        image: event.date,
      );
      emit(SoliqIdCompareFaceSuccessState(
        response: response,
      ));
    } on Object catch (e) {
      log(e.toString());
    }
  }
}
